"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var actiontype_1 = require("./actiontype");
var pending_value_1 = require("./pending_value");
var util_1 = require("./util");
var lodash_1 = __importDefault(require("lodash"));
function connectPod(enhancerContext, dispatch, storeState, reducer, reducerState) {
    var e_1, _a;
    var podPath = util_1.findPath(storeState, reducerState);
    if (enhancerContext.pathSet.includes(podPath)) {
        console.error("Warning: pod reducer at '" + podPath + "' has already been connected.");
    }
    else {
        enhancerContext.pathSet.push(podPath);
    }
    var props = reducer.getProps();
    var reducerFunction = reducer.getBoundFunc();
    var actionSet = props.actionSet;
    var _loop_1 = function (action) {
        reducerFunction[action.getType()] = action.assign(function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return dispatch(action.getBoundFunc().apply(void 0, __spread(args)));
        });
    };
    try {
        for (var _b = __values(Object.values(actionSet)), _c = _b.next(); !_c.done; _c = _b.next()) {
            var action = _c.value;
            _loop_1(action);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
    props.path = podPath;
    props.connected = true;
    return (reducerState instanceof pending_value_1.PendingValue
        ? reducerState.valueOf()
        : reducerState);
}
function init(enhancerContext, store) {
    var e_2, _a;
    store.dispatch({
        type: actiontype_1.INTERNAL_ACTION_TYPES.init,
        init: function (initialState) {
            return new pending_value_1.PendingValue(initialState);
        }
    });
    var postInitState = store.getState();
    store.dispatch({
        type: actiontype_1.INTERNAL_ACTION_TYPES.connect,
        connect: function (reducerState, reducer) {
            return connectPod(enhancerContext, store.dispatch, postInitState, reducer, reducerState);
        }
    });
    store.dispatch({
        type: actiontype_1.INTERNAL_ACTION_TYPES.registerTrackers,
        register: function (tracked) {
            var path = tracked().getProps().path;
            if (!enhancerContext.trackedPaths.has(path)) {
                enhancerContext.trackedPaths.set(path, undefined);
            }
        }
    });
    if (enhancerContext.trackedPaths.size) {
        var currentStoreState = store.getState();
        var _loop_2 = function (path) {
            enhancerContext.trackedPaths.set(path, lodash_1.default.get(currentStoreState, path));
            store.subscribe(function () {
                var currentState = lodash_1.default.get(store.getState(), path);
                var previousState = enhancerContext.trackedPaths.get(path);
                if (currentState !== previousState) {
                    enhancerContext.trackedPaths.set(path, currentState);
                    store.dispatch({
                        type: actiontype_1.INTERNAL_ACTION_TYPES.tracker,
                        currentState: currentState,
                        previousState: previousState,
                        path: path
                    });
                }
            });
        };
        try {
            for (var _b = __values(enhancerContext.trackedPaths.keys()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var path = _c.value;
                _loop_2(path);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
    }
}
function reduxPodEnhancer(options) {
    if (options === void 0) { options = {}; }
    var context = {
        pathSet: [],
        trackedPaths: new Map()
    };
    return function (next) { return function () {
        var enhancer = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            enhancer[_i] = arguments[_i];
        }
        var store = next.apply(void 0, __spread(enhancer));
        if (!options.initializeOn) {
            init(context, store);
            return store;
        }
        return __assign(__assign({}, store), { dispatch: function (action) {
                if (options.initializeOn(action)) {
                    init(context, store);
                }
                return store.dispatch(action);
            } });
    }; };
}
exports.reduxPodEnhancer = reduxPodEnhancer;
