"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * This abstract super class is responsible for producing a bound function through the
 * root function and bound function members defined by the child class. This can be extended
 * by any class which needs to produce a specific function with specific assigned members.
 *
 * Used by the `PodReducer` and `ProxiedAction` classes.
 */
var FunctionProducer = /** @class */ (function () {
    /**
     * Class constructor which accepts the origin root function from the child class.
     *
     * @param unboundFunction The origin unbound root function.
     */
    function FunctionProducer(unboundFunction) {
        this.unboundFunction = unboundFunction;
    }
    /**
     * Returns the bound function, and if it has yet to be assigned the bound function
     * members, assigns the function members beforehand.
     */
    FunctionProducer.prototype.getBoundFunc = function () {
        if (!this.boundFunction) {
            this.boundFunction = this.assign(this.unboundFunction);
        }
        return this.boundFunction;
    };
    /**
     * A util. function to assign the bound function members supplied by the child class to
     * any other obj. the consumer provides.
     *
     * @param to The obj. to assign the bound function members to.
     */
    FunctionProducer.prototype.assign = function (to) {
        return Object.assign(to, this.boundFunctionMembers());
    };
    return FunctionProducer;
}());
exports.FunctionProducer = FunctionProducer;
