"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("./util");
var state_1 = require("./state");
var actionset_1 = require("./actionset");
var tracker_1 = require("./tracker");
var effect_1 = require("./effect");
var PodMethods = /** @class */ (function () {
    function PodMethods(instance) {
        this.instance = function () { return instance; };
    }
    PodMethods.prototype.path = function () {
        return this.props().path;
    };
    PodMethods.prototype.connected = function () {
        return this.props().connected;
    };
    PodMethods.prototype.actionSet = function () {
        return this.props().actionSet;
    };
    PodMethods.prototype.props = function () {
        return this.instance().getProps();
    };
    PodMethods.prototype.instance = function () {
        return undefined;
    };
    PodMethods.prototype.mapState = function (storeState) {
        var selector = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            selector[_i - 1] = arguments[_i];
        }
        return typeof selector[0] === 'function'
            ? state_1.mapStateFromFunction(selector[0], storeState, this.path(), [])
            : state_1.mapStateFromPath(storeState, this.path(), typeof selector[0] !== 'string' ? undefined : selector);
    };
    PodMethods.prototype.on = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var actionSet = util_1.isActionSet(args[0]);
        if (args.length === 0 || (!actionSet && typeof args[0] !== 'string')) {
            throw new Error("Invalid argument of type " + typeof args[0] + "; expected action set obj. or action type string.");
        }
        return this.instance().chain(actionSet
            ? {
                actionSet: actionset_1.chainedActionSet(this.instance, this.actionSet(), args[0])
            }
            : {
                effectChain: effect_1.actionTypeEffect(typeof args[0] === 'string' ? [args[0]] : args[0], args[1])
            });
    };
    PodMethods.prototype.track = function (toTrack, tracker) {
        var pod = util_1.isPod(toTrack);
        var action = !pod && util_1.isProxiedAction(toTrack);
        if (!pod && !action) {
            throw new Error("Invalid tracker reference; expected pod or pod action, but got: " + typeof toTrack);
        }
        return this.instance().chain(pod
            ? {
                trackers: new Map(this.props().trackers).set(toTrack.instance, tracker),
                effectChain: tracker_1.podTrackerEffect(toTrack, tracker)
            }
            : {
                effectChain: tracker_1.actionTrackerEffect(toTrack, tracker)
            });
    };
    PodMethods.prototype.reduce = function (effect) {
        return this.instance().chain({
            effectChain: effect_1.internalReducerEffect(this.actionSet(), effect)
        });
    };
    PodMethods.prototype.reduceAny = function (effect) {
        return this.instance().chain({
            effectChain: effect
        });
    };
    PodMethods.prototype.extend = function (extendedState) {
        var initialState = this.props().initialState;
        if (extendedState !== undefined &&
            typeof extendedState !== typeof initialState) {
            throw new Error("An extended initial state must be of the same type as the existing initial state declaration.");
        }
        return this.instance().chain(typeof extendedState !== undefined
            ? {
                initialState: typeof extendedState === 'string' ||
                    typeof extendedState === 'boolean' ||
                    typeof extendedState === 'number' ||
                    typeof extendedState === 'bigint'
                    ? { initialState: extendedState }
                    : __assign(__assign({}, initialState), extendedState)
            }
            : {});
    };
    return PodMethods;
}());
exports.PodMethods = PodMethods;
