"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var immer_1 = __importDefault(require("immer"));
var actiontype_1 = require("./actiontype");
var function_producer_1 = require("./function_producer");
var methods_1 = require("./methods");
var properties_1 = require("./properties");
var util_1 = require("./util");
/**
 * The root pod reducer class as a `FunctionProducer`. The main purpose of this class is
 * to initialize the root reducer function as a handle for incoming internal actions and to
 * propagate the incoming state/action objects through the stateful effect chain.
 *
 * It's also responsible for binding the root reducer function with the external methods
 * provided by the `PodMethods` class, along with the proxied action creators chained onto
 * the pod.
 *
 * Used by the root `pod` function to instantiate new pod instances.
 */
var PodReducer = /** @class */ (function (_super) {
    __extends(PodReducer, _super);
    /**
     * Pod reducer class constructor providing the root reducer function to the
     * function producer super class.
     *
     * @param props The initial or chained `PodProperties` object.
     */
    function PodReducer(props) {
        var _this = _super.call(this, function (state, action) {
            if (state === void 0) { state = props.initialState; }
            return typeof action !== 'object' || !action.type
                ? state
                : _this.handleAction(state, action);
        }) || this;
        _this.props = props;
        return _this;
    }
    /**
     * The first action handler method for all incoming actions. Checks for internal
     * actions, a match on any of the proxied actions and otherwise propagates a drafted
     * state obj. and the incoming action through the stateful effect chain.
     *
     * @param state The incoming state obj. from the redux store.
     * @param action The incoming action obj.
     */
    PodReducer.prototype.handleAction = function (state, action) {
        var e_1, _a;
        var _this = this;
        switch (action.type) {
            case actiontype_1.INTERNAL_ACTION_TYPES.init:
                return !this.props.connected ? action.init(state) : state;
            case actiontype_1.INTERNAL_ACTION_TYPES.connect:
                return !this.props.connected ? action.connect(state, this) : state;
            case actiontype_1.INTERNAL_ACTION_TYPES.registerTrackers:
                if (this.props.trackers) {
                    try {
                        for (var _b = __values(this.props.trackers.keys()), _c = _b.next(); !_c.done; _c = _b.next()) {
                            var pod = _c.value;
                            action.register(pod);
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                }
                return state;
        }
        return immer_1.default(state, function (draft) {
            var e_2, _a;
            try {
                for (var _b = __values(Object.values(_this.props.actionSet)), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var proxiedAction = _c.value;
                    if (proxiedAction.match(action)) {
                        return proxiedAction.process(draft, action);
                    }
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_2) throw e_2.error; }
            }
            return _this.props.effectChain
                ? _this.props.effectChain(draft, action)
                : draft;
        });
    };
    /**
     * Retreives the `PodProperties` obj. corresponding to the pod reducer.
     */
    PodReducer.prototype.getProps = function () {
        return this.props;
    };
    /**
     * Chains the reducer with a new reducer instance with an updated set of properties.
     *
     * @param propsToMerge A partial property obj. to merge into the chained instance.
     */
    PodReducer.prototype.chain = function (propsToMerge) {
        if (propsToMerge === void 0) { propsToMerge = {}; }
        if (propsToMerge.effectChain) {
            propsToMerge.effectChain = util_1.mergeEffects(this.props.effectChain, propsToMerge.effectChain);
        }
        return new PodReducer(new properties_1.PodProperties(__assign(__assign({}, this.props), propsToMerge))).getBoundFunc();
    };
    /**
     * Provies an obj. of bound `PodMethods` members along with external proxied action
     * handlers to the super `FunctionProducer` class, which is then applied directly
     * to the root reducer funuction.
     */
    PodReducer.prototype.boundFunctionMembers = function () {
        var e_3, _a;
        var methods = new methods_1.PodMethods(this);
        var boundFunctions = {};
        Object.keys(methods_1.PodMethods.prototype).forEach(function (key) {
            boundFunctions[key] = function () {
                var _a;
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return (_a = methods[key]).call.apply(_a, __spread([methods], args));
            };
        });
        if (this.props.actionSet) {
            try {
                for (var _b = __values(Object.values(this.props.actionSet).values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var action = _c.value;
                    boundFunctions[action.getType()] = action.getBoundFunc();
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_3) throw e_3.error; }
            }
        }
        return boundFunctions;
    };
    return PodReducer;
}(function_producer_1.FunctionProducer));
exports.PodReducer = PodReducer;
